#include "stm32f4xx.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_exti.h"
#include "misc.h"
#include "stm32f4xx_syscfg.h"
#include "stm32f4xx_tim.h"
#include "math.h"
#include "stm32f4xx_iwdg.h"
#include "stm32f4xx_adc.h"


uint8_t    SkeepCicle, IsBeforeCollect, IsFirstFlash, OutPutChanel;
uint16_t   AngleHandCorrection;
uint32_t   TurnsOfMinute, AngleDuration, LowLevelDuration, HiLevelDuration, CollectTime, EthalonDuration,
           TimeBeforeCollect, PressCalibrValue, PressCurrentValue;
float      BeginingAngle, CurrentAngle, OneDegreeDuration, AngleCorrection, PressCorrection, A, B;

uint16_t  readADC1(u8 channel){
   ADC_RegularChannelConfig(ADC1, channel, 1, ADC_SampleTime_3Cycles);
   ADC_SoftwareStartConv(ADC1);
   while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
   return ADC_GetConversionValue(ADC1);
}

uint16_t GetCurrentPressure(u8 channel){
	uint16_t currentpressure;
	currentpressure = readADC1(channel);
		if (currentpressure >= 230)
			currentpressure = 256;
		if (currentpressure <= 30)
			currentpressure = 256;
		if (currentpressure == 256)
			GPIO_WriteBit(GPIOA,GPIO_Pin_7, Bit_RESET);//   
		else
			GPIO_WriteBit(GPIOA,GPIO_Pin_7, Bit_SET);//   
	return currentpressure;
}

//     
void EXTI3_IRQHandler(void) {
	if(EXTI_GetITStatus(EXTI_Line3)!=RESET)	{
		EXTI_ClearITPendingBit(EXTI_Line3);
		if (IsBeforeCollect == 1){
			OutPutChanel = 2;
			return;
		}
		NVIC_DisableIRQ(EXTI_Line3);
		IWDG_ReloadCounter();
		TIM_Cmd(TIM5, DISABLE);
		if(GPIO_ReadInputDataBit(GPIOE,GPIO_Pin_3)==1) {
			//  ,   
			if (SkeepCicle == 0)
			{	LowLevelDuration  = TIM5->CNT;
			    OutPutChanel = 1;
				IsBeforeCollect = 1;
				TurnsOfMinute = EthalonDuration / LowLevelDuration * 30;
				HiLevelDuration = round(LowLevelDuration * 2.71134);
				OneDegreeDuration = (float)(LowLevelDuration + HiLevelDuration)/180;
				if (LowLevelDuration > 9000)
					CollectTime = 9000;//  4.5
				else
					CollectTime = round(OneDegreeDuration * 56.0);
				// 
				A = powf(TurnsOfMinute,2);
				B = TurnsOfMinute;
				if (TurnsOfMinute >= 2820) {
					A = A * 0.0000005;
					B = B * 0.00655;
					CurrentAngle = B - A + 16.75;
				}else{
					if (TurnsOfMinute >= 2000) {
						A = A * 0.0000024;
						B = B * 0.0022;
						CurrentAngle = A + B + 6.0;
					}else{
						if (TurnsOfMinute >= 500) {
							A = A * 0.000003;
							B = B * 0.0005;
							CurrentAngle = A + B + 7.0;
						}else{
							CurrentAngle = BeginingAngle;
						}
					}
				}
				//  
				if (AngleHandCorrection > 63){
					AngleCorrection = (float)(AngleHandCorrection - 63)*0.5;
					CurrentAngle = CurrentAngle + AngleCorrection;
				}else{
					AngleCorrection = (float)(63 - AngleHandCorrection)*0.5;
					if (CurrentAngle >= AngleCorrection)
						CurrentAngle = CurrentAngle - AngleCorrection;
					else
						CurrentAngle = BeginingAngle;
				}
				//   
				if (PressCalibrValue != 256){
					if (PressCurrentValue > 90)
						PressCurrentValue = 90;
					PressCorrection = (float)PressCurrentValue / 300;
					CurrentAngle = CurrentAngle * (1 + PressCorrection);
				}
				AngleDuration = round(CurrentAngle * OneDegreeDuration);
				//    
				TimeBeforeCollect = HiLevelDuration - AngleDuration - CollectTime;
				TIM5->CNT = 0;
				TIM5->ARR = TimeBeforeCollect;
				NVIC_EnableIRQ(TIM5_IRQn);
				TIM_Cmd(TIM5, ENABLE);
			}
		}else{
			//  ,   
			GPIO_WriteBit(GPIOA, GPIO_Pin_0|GPIO_Pin_1, Bit_RESET);
			TIM5->CNT = 0;
			TIM5->ARR = EthalonDuration;
			NVIC_DisableIRQ(TIM5_IRQn);
			TIM_Cmd(TIM5, ENABLE);
			AngleHandCorrection = readADC1(ADC_Channel_2) / 2;
			if (PressCalibrValue < 256){
				PressCurrentValue = GetCurrentPressure(ADC_Channel_3);
				if (PressCurrentValue > PressCalibrValue)
					PressCalibrValue = PressCurrentValue;
				PressCurrentValue = PressCalibrValue - PressCurrentValue;
			}
			SkeepCicle = 0;
		}
		NVIC_EnableIRQ(EXTI_Line3);
	}
}


void TIM5_IRQHandler(void){
	if(TIM_GetITStatus(TIM5, TIM_IT_Update)!=RESET){
		TIM_Cmd(TIM5, DISABLE);
		if (IsBeforeCollect == 1){
			//   
			IsBeforeCollect = 0;
			TIM5->CNT = 0;
			TIM5->ARR = CollectTime;
			TIM_Cmd(TIM5, ENABLE);
			//          
			//       
			if (IsFirstFlash == 1){
				IsFirstFlash = 0;
				return;
			}
			// 
			switch (OutPutChanel) {
				case 1:
					GPIO_WriteBit(GPIOA, GPIO_Pin_0, Bit_SET);
					break;
				case 2:
					GPIO_WriteBit(GPIOA, GPIO_Pin_1, Bit_SET);
					break;
				default:
					break;
			}
		}else{
			// ,  
			GPIO_WriteBit(GPIOA, GPIO_Pin_0|GPIO_Pin_1, Bit_RESET);
			NVIC_DisableIRQ(TIM5_IRQn);
		}
		TIM_ClearITPendingBit(TIM5, TIM_IT_Update);
	}
}

void init_interrupts(){
	SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOE,EXTI_PinSource3);
    EXTI_InitTypeDef exti;
    exti.EXTI_Line = EXTI_Line3;
    exti.EXTI_Mode = EXTI_Mode_Interrupt;
    exti.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
    exti.EXTI_LineCmd = ENABLE;
    EXTI_Init(&exti);

    NVIC_InitTypeDef nvic;
    nvic.NVIC_IRQChannel = EXTI3_IRQn;
    nvic.NVIC_IRQChannelPreemptionPriority = 0;
    nvic.NVIC_IRQChannelSubPriority = 0;
    nvic.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&nvic);
    NVIC_EnableIRQ(EXTI3_IRQn);
}

void iwdg_init(void) {
	RCC_LSICmd(ENABLE);
	while (RCC_GetFlagStatus(RCC_FLAG_LSIRDY) == RESET);
	IWDG_WriteAccessCmd(0x5555);
	IWDG_SetPrescaler(IWDG_Prescaler_32);
	IWDG_SetReload(1250);
	IWDG_ReloadCounter();
	IWDG_Enable();
}

void set_timer_paramms(TIM_TypeDef* TIMx, uint32_t period){
	TIM_SetCounter(TIMx, 0);
	TIM_TimeBaseInitTypeDef timer;
	TIM_TimeBaseStructInit(&timer);
	timer.TIM_Prescaler = 41;
	timer.TIM_Period = period;
	timer.TIM_CounterMode = TIM_CounterMode_Up;
	timer.TIM_ClockDivision = 0;
	TIM_TimeBaseInit(TIMx, &timer);
}

void init_timers(){
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM5, ENABLE);
    set_timer_paramms(TIM5, EthalonDuration);
	TIM_ITConfig(TIM5,TIM_IT_Update,ENABLE);
	NVIC_DisableIRQ(TIM5_IRQn);
	TIM_Cmd(TIM5, DISABLE);
}

void init_pins(){
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA|RCC_AHB1Periph_GPIOE, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG,ENABLE);
	GPIO_WriteBit(GPIOA, GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_6|GPIO_Pin_7, Bit_RESET);
	GPIO_InitTypeDef gpio;

	GPIO_StructInit(&gpio);
	gpio.GPIO_Pin = GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_6|GPIO_Pin_7;
	gpio.GPIO_Mode = GPIO_Mode_OUT;
	gpio.GPIO_OType = GPIO_OType_PP;
	gpio.GPIO_Speed = GPIO_Speed_2MHz;
	gpio.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOA, &gpio);

	GPIO_StructInit(&gpio);
	gpio.GPIO_Pin = GPIO_Pin_2|GPIO_Pin_3;
	gpio.GPIO_Mode = GPIO_Mode_AN;
	GPIO_Init(GPIOA, &gpio);

	GPIO_StructInit(&gpio);
	gpio.GPIO_Pin = GPIO_Pin_3;
	gpio.GPIO_Speed = GPIO_Speed_2MHz;
	gpio.GPIO_PuPd = GPIO_PuPd_NOPULL;
	gpio.GPIO_Mode = GPIO_Mode_IN;
	GPIO_Init(GPIOE, &gpio);

	GPIO_WriteBit(GPIOA, GPIO_Pin_6|GPIO_Pin_7, Bit_SET);//  
}

void init_adc() {
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
	ADC_DeInit();

    ADC_CommonInitTypeDef adc_init;
    ADC_CommonStructInit(&adc_init);
    adc_init.ADC_Mode = ADC_Mode_Independent;
    adc_init.ADC_Prescaler = ADC_Prescaler_Div2;
    ADC_CommonInit(&adc_init);

    ADC_InitTypeDef ADC_InitStructure;
    ADC_StructInit(&ADC_InitStructure);
    ADC_InitStructure.ADC_ScanConvMode = DISABLE;
    ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
    ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConvEdge_None;
    ADC_InitStructure.ADC_Resolution = ADC_Resolution_8b;
    ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
    ADC_InitStructure.ADC_NbrOfConversion = 1;
    ADC_Init(ADC1, &ADC_InitStructure);
    ADC_Cmd(ADC1, ENABLE);
}

int main(void){
	EthalonDuration = 269444;
	SkeepCicle = 1;
	IsFirstFlash = 1;
	IsBeforeCollect = 0;
	AngleHandCorrection = 0;
	PressCorrection = 0;
	BeginingAngle = 5.0;
	OutPutChanel = 0;
	SystemInit();
	init_pins();
	init_timers();
	init_interrupts();
	init_adc();
	iwdg_init();
	PressCalibrValue = GetCurrentPressure(ADC_Channel_3);// 
    while(1) {
    }
}

